 LIST P=16F628, R=DEC  ; Use the PIC16F628 and decimal system 

;=========================================================================================
;Program: Programovatelne pocitadlo pro vypinani soustruhu
;Autor: Jan Matuska, walda@starhill.org, toho casu FSI VUT v Brne 
;Datum: 2.7.2005
;Komentar: Program je opet velmi primitivni, pameti dost, pozadovanych funkci malo, takze
;          neni nutne se snazit o otimalizaci za kazdou cenu. Zabere asi 26% pameti PICu.
;=========================================================================================

 #include "P16F628.INC"        ; Include header file 

; Config pro zapojeni bez krystalu a bez externiho MCLR 
 __config  _INTRC_OSC_NOCLKOUT &_LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_OFF & _MCLRE_OFF 


;--- deklarace promennych ---
temp        equ 20h           ;pomocny registr
TMP_W       equ 21h           ;uschovani W pri preruseni
TMP_S       equ 22h           ;uschovani STATUS pri preruseni
TMP_PCL     equ 23h           ;uschovani horni casti PC latch
pozice      equ 24h           ;pozice rozsvicene 7-segmentovky
CIS1_HI     equ 25h           ;vstup pro odecitani 16-ti bitovych cisel
CIS1_LO     equ 26h           ;vstup pro odecitani 16-ti bitovych cisel
CIS2_HI     equ 27h           ;vstup pro odecitani 16-ti bitovych cisel
CIS2_LO     equ 28h           ;vstup pro odecitani 16-ti bitovych cisel
NUMHI       equ 29h           ;vystup rozdilu, vstup prevodu bin na BCD
NUMLO       equ 2Ah           ;vystup rozdilu, vstup prevodu bin na BCD
D3          equ 2Bh           ;BCD hodnota ticicu, nerobrazuje se, jen vypocet 
D2          equ 2Ch           ;BCD hodnota stovek, nerobrazuje se, jen vypocet  
D1          equ 2Dh           ;BCD hodnota desitek, nerobrazuje se, jen vypocet  
D0          equ 2Eh           ;BCD hodnota jednotek, nerobrazuje se, jen vypocet  
hi          equ 2Fh           ;vystup prevodu z BCD na bin
lo          equ 30h           ;vystup prevodu z BCD na bin
tenk        equ 31h           ;prevodu z BCD na bin, toto se nevyuziva, je to stale 0
pau         equ 32H           ;cekaci smycky
count       equ 33H           ;cekaci smycky
d100        equ 34h           ;kod zasilany na port k segmisegmentovce, rad 100
d10         equ 35h           ;kod zasilany na port k segmisegmentovce, rad 10
d1          equ 36h           ;kod zasilany na port k segmisegmentovce, rad 1
cnthi       equ 37h           ;pocitadlo prichozich vstupu, stezejni registr
cntlo       equ 38h           ;pocitadlo prichozich vstupu, stezejni registr
stophi      equ 39h           ;hodnota, kdy se ma stroj vypnout
stoplo      equ 3Ah           ;hodnota, kdy se ma stroj vypnout

; registr stavu
stav        equ 3Bh
countdown   equ 0h            ;L=normal H=countdown
sep_spinac  equ 1h            ;L=tlacitko neni sepnuto H=tlacitko je sepnuto
sep_k_up    equ 2h            ;L=tlacitko neni sepnuto H=tlacitko je sepnuto
cutzeros    equ 3h            ;L=nuly budou orezane H=nebude se zasahovat do vypisu



;--- nastaveni jmen pro vstupy a vystupy ---
; vystupy z PICu
seg1        equ 6h            ;rozsviceni segmentu radu 1
seg10       equ 5h            ;rozsviceni segmentu radu 10
seg100      equ 7h            ;rozsviceni segmentu radu 100
rele        equ 1h            ;sepnuti rele vypinajici stroj
; vstupy do PICu
spinac      equ 4h            ;vstup spinace, negovany
k_clr       equ 3h            ;tlacitko CLEAR (VYNULOVAT), negovane
k_set       equ 2h            ;tlacitko SET (NASTAVIT), negovane
k_up        equ 0h            ;tlacitko UP (NAHORU/ODECET), negovane
 

;--- nastaveni konstatnt --- 
; cas prepinani segmentu (0-255): t = 65.535ms - seg_selay * 0.255ms
;                                 seg_delay = ( 65.535 - t ) / 0.255
 constant seg_delay=246


;--- nastaveni dat v EEPROM ---
 org 0x2100  
; horni a spodni byte nastaveni citace, v tomto pripade 1
 de 0,9
; adresa hi (lo je +1) bytu aktualniho citace od 4h do 7Eh
 de 4
 org 0x2104
; vychozi nastaveni citace na 0
 de 0,0 



; Reset procesoru --------------------------------------------------------------

              org 0x0000                ;zacatek programu
              goto Init                 ;skok na pocatecni inicializaci

              org 0x0004                ;vektor preruseni
              goto Interrupt            ;skok na obsluhu preruseni



; Obsluha preruseni od TMR1 ----------------------------------------------------

Interrupt     movwf TMP_W               ;ulozeni hodnot registru W a STATUS
              movf STATUS,W             ;prohodi nibble STATUS a ulozi do W
              clrf STATUS               ;nastaveni banky 0
              movwf TMP_S               ;ulozit STATUS
              movf PCLATH,W             ;ulozit PC Latch
              movwf TMP_PCL             ;do pomocne promenne
              clrf PCLATH               ;smazat PC Latch
              movlw seg_delay           ;nastaveni horniho byte casovace 
              movwf TMR1H               ;kopie do registru TMR1H
              ;clrwdt                   ;da pejskovi nazrat
              incf pozice               ;inkrementace pozice displeje
              movlw 3                   ;3 do W
              subwf pozice,w            ;odecteni pozice do W, vysledek do W
              btfsc STATUS,Z            ;je pozice=3?
              clrf pozice               ;ano, pozice=0
              movfw pozice              ;ne, pozice do W
              addwf PCL,f               ;pricist pozici k PC counteru
              goto i_1                  ;skok, kdyz pozice=0
              goto i_2                  ;skok, kdyz pozice=1
              goto i_3                  ;skok, kdyz pozice=2
i_1           movfw d1                  ;kod cisla pro rad 1 do W
              bcf PORTB,seg100          ;vypne cislici radu 100
              bcf PORTB,seg10           ;vypne cislici radu 100
              bsf PORTB,seg1            ;zapne cislici radu 100
              movwf PORTA               ;W na port A, tedy zobrazeni cisla 
              goto InterruptEnd         ;skok na konec preruseni
i_2           movfw d10                 ;kod cisla pro rad 10 do W
              bcf PORTB,seg100          ;vypne cislici radu 100
              bsf PORTB,seg10           ;zapne cislici radu 100
              bcf PORTB,seg1            ;vypne cislici radu 100
              movwf PORTA               ;W na port A, tedy zobrazeni cisla 
              goto InterruptEnd         ;skok na konec preruseni
i_3           movfw d100                ;kod cisla pro rad 100 do W
              bsf PORTB,seg100          ;zapne cislici radu 100
              bcf PORTB,seg10           ;vypne cislici radu 100
              bcf PORTB,seg1            ;vypne cislici radu 100
              movwf PORTA               ;W na port A, tedy zobrazeni cisla 
InterruptEnd  clrf STATUS               ;smaze status, tzn. bank na 0
              bcf PIR1,TMR1IF           ;shozeni priznaku preteceni TMR1
              movf TMP_PCL,W            ;docasny PCLatch do W
              movwf PCLATH              ;obnova PCLATH
              movf TMP_S,W              ;prohodi nibble TMP_S a ulozi do W
              movwf STATUS              ;obnova STATUS
              swapf TMP_W,F             ;prohodi nibble TMP_W a ulozi do W
              swapf TMP_W,W             ;obnova W
              retfie                    ;navrat z preruseni



; Dekoder BCD na 7-segment  vstup je hodnota z W -------------------------------

Decoder       bcf W,7                   ;smazani bitu 7, ochrana proti preteceni
              bcf W,6                   ;smazani bitu 6, ochrana proti preteceni
              bcf W,5                   ;smazani bitu 5, ochrana proti preteceni
              bcf W,4                   ;smazani bitu 4, ochrana proti preteceni
              addwf PCL,f               ;pricteni hodnoty z W k PC counteru
              retlw b'00000100'         ;0   0
              retlw b'00011111'         ;1   1
              retlw b'01000001'         ;2   2
              retlw b'00010001'         ;3   3
              retlw b'00011010'         ;4   4
              retlw b'10010000'         ;5   5
              retlw b'10000000'         ;6   6
              retlw b'00011101'         ;7   7
              retlw b'00000000'         ;8   8
              retlw b'00010000'         ;9   9
              retlw b'11011011'         ;-  10
              retlw b'11010111'         ;_  11
              retlw b'11011111'         ;   12
              retlw b'11000000'         ;E  13
              retlw b'10001011'         ;n  14
              retlw b'00000011'         ;d  15



; Inicializace procesoru -----------------------------------------------------------------

Init          clrf pozice               ;pozice na prvni cislicovku
              bcf STATUS,RP0            ;bank 0
              movlw 7                   ;7 do W
              movwf CMCON               ;CMCON=7 set comparators off   
              bsf STATUS,RP0            ;bank 1
	            movlw 0h                  ;0 do W
              movwf VRCON               ;W do VRCON      
              movlw b'00001111'         ;pull-up, preddelicka na WDT, cca 2.3sec
              movwf OPTION_REG          ;W do option
              ; --- Nastaveni portu ---
              movlw b'00100000'         ;portA 1=in, 0=out
              movwf TRISA               ;portA
              movlw b'00011101'         ;portB 1=in, 0=out
              movwf TRISB               ;portB
              bcf STATUS,RP0            ;bank 0
              clrf PORTA                ;smazat port A
              clrf PORTB                ;smazat port B
              ; --- Nastaveni Timeru1 jako zdroje preruseni ---
              movlw seg_delay           ;nastaveni horniho byte casovace na konst.
              movwf TMR1H               ;W do tmrh1
              bcf INTCON, GIE           ;zakazat preruseni preventivne
              call Delay1Sec            ;cas na ustaleni elektrickych velicin
              movlw b'00000101'         ;nastaveni timeru1
              movwf T1CON               ;v registru T1CON
              bsf INTCON,PEIE           ;1=povoleno peruen pi od periferie
              bsf STATUS,RP0            ;bank 1
              bsf PIE1,TMR1IE           ;preruseni od periferie - timer1
              bcf STATUS,RP0            ;bank 0
              ; --- Nacteni nastavenych hodnot z EE do stophi a stoplo ---
              movlw 0h                  ;adresa 0h do W
              call ReadEE               ;precist z adresy 0
              movwf stophi              ;ulozit do stophi
              movlw 1h                  ;adresa 1h do W
              call ReadEE               ;precist z adresy 1
              movwf stoplo              ;ulozit do stoplo
                    ; --- Nacteni aktualnich hodnot z EE do cnthi a cntlo ---
              movlw 2h                  ;adresa 2h do W
              call ReadEE               ;precist z adresy 2
              movwf temp                ;ulozit do temp, ve vysledek W to zustane
              call ReadEE               ;precist z adresy dle W
              movwf cnthi               ;ulozit do cnthi
              incf temp,W               ;inkrementovat temp a ulozit do W
              call ReadEE               ;precist z adresy dle vysledku ve W
              movwf cntlo               ;ulozit do cntlo
                    ; --- Vychozi stav precovnich registru ---
              clrf stav                 ;smazat stavovy registr
              call Zobraz               ;prevod z cnthi a cntlo na BCD a displej
              clrf pozice               ;pozice na prvni cislicovku
              bsf INTCON, GIE           ;povolit preruseni
                    ; --- Konec inicializace ---
              goto MainLoop             ;skok na hlavni cast programu




; Hlavni smycka programu -----------------------------------------------------------------

MainLoop      btfss PORTB,spinac        ;je spinac sepnuty?
              call SepnutSpinac         ;ano, zavolani SepnutSpinac
              btfss PORTB,k_clr         ;je tlacitko CLEAR sepnute?
              call SepnutClr            ;ano, zavolani SepnutClr
              btfss PORTB,k_set         ;je tlacitko SET sepnute?
              call SepnutSet            ;ano, zavolani SepnutSet
              btfss PORTB,k_up          ;je tlacitko UP sepnute?
              call SepnutUp             ;ano, zavolani SepnutUp
              btfsc PORTB,spinac        ;je spinac sepnuty?
              call OutSpinac            ;ne neni, zavolani OutSpinac  
              btfsc PORTB,k_up          ;je tlacitko UP sepnute?
              call OutKup               ;ne neni, zavolani OutKup
              goto MainLoop             ;skok na zacatek smycky MainLoop


OutSpinac     call Delay100ms           ;pocka 100ms
              btfsc PORTB,spinac        ;je spinac sepnuty?
              bcf stav,sep_spinac       ;ne neni, smaze priznak sepnuteho spinace
              return                    ;ano i ne, navrat z OutSpinac 


OutKup        call Delay100ms           ;pocka 100ms
              btfsc PORTB,k_up          ;je tlacitko UP sepnute?
              bcf stav,sep_k_up         ;ne neni, smaze priznak sepnuteho tlacitka UP
              return                    ;ano i ne, navrat z OutKup


SepnutSpinac  btfsc stav,sep_spinac     ;je nastaven priznak sepnuteno spinace?
              return                    ;ano je, navrat ze SepnutSpinac
              call Delay100ms           ;pocka 200ms
              call Delay100ms
              btfsc PORTB,spinac        ;je sepnut spinac?
              return                    ;ne neni, navrat ze SepnutSpinac
              bsf stav,sep_spinac       ;ano je, nastaveni priznaku sepnuteho spinace
              incf cntlo,f              ;inkrementace cntlo a ulozeno do cntlo
              btfsc STATUS,Z            ;doslo k preteceni? Tedy 255+1=0?
              incf cnthi,f              ;ano doslo, inkrementace cnthi
              call Zobraz               ;zobraz na displeji
              call WriteCnt2EE          ;zapis aktualni stav citace do EEPROM
              ; --- Porovnani citacu stop a cnt ---
              movfw stophi              ;stophi do W
              subwf cnthi,w             ;odecti W od cnthi
              btfss STATUS,Z            ;je stophi=cnthi ?
              goto SepnutSp1            ;ne neni, skok na SepnutSp1
              movfw stoplo              ;ano je, stoplo do W
              subwf cntlo,w             ;odecti W od cntlo
              btfss STATUS,Z            ;je stoplo=cntlo ?
              goto SepnutSp1            ;ne neni, skok na SepnutSp1 
              call ClearCounter         ;smaze citac, ulozi do EEPROM nulu atd..
              call MachineStop          ;citac docital, zastavme stroje
SepnutSp1     return                    ;navrat z SepnutSpinac


SepnutUp      btfsc stav,sep_k_up       ;je nastaven priznak sepnuteno tlacitka UP?
              return                    ;ano je, navrat ze SepnutUp
              call Delay100ms           ;ne neni, pocka 100ms
              btfsc PORTB,k_up          ;je opravdu stisknuto tlacitko UP?
              return                    ;ne neni, navrat ze SepnutUp
              bsf stav,sep_k_up         ;ano je, nastaveni priznaku sepnuteho tlacitka UP
              btfss stav,countdown      ;je nastaven priznak zobrazeni countdown citace?
              goto SepnutUp1            ;ne neni, skok na SepnutUp1
              bcf stav,countdown        ;ano je, nastavi priznak zobrazeni countdownu
              call Zobraz               ;posle hned na displej
              return                    ;navrat ze SepnutUp
SepnutUp1     bsf stav,countdown        ;smaze priznak zobrazeni countdownu
              call Zobraz               ;posle hned na displej
              return                    ;navrat ze SepnutUp


SepnutClr     call Delay100ms           ;pocka 100ms
              btfsc PORTB,k_clr         ;je tlacitko CLEAR opravdu stisknuto?
              return                    ;ne neni, navrat ze SepnutClr
              call ClearCounter         ;zavolani smazani citace
SepnutClr1    btfss PORTB,k_clr         ;je tlacitko CLEAR stisknuto
              goto SepnutClr1           ;ano je, skok na SepnutClr1
              call Delay100ms           ;ne neni, pocka 100ms
              btfss PORTB,k_clr         ;je tlacitko CLEAR stisknuto?
              goto SepnutClr1           ;ano je, skok na SepnutClr1
              return                    ;navrat z SepnutClr


SepnutSet     call Delay100ms           ;pocka 100ms
              btfsc PORTB,k_set         ;je tlacitko SET opravdu stisknuto?
              return                    ;ne neni, navrat ze SepnutSet
SepnutSet1    btfss PORTB,k_set         ;ano je, je je tlacitko SET stisknuto?
              goto SepnutSet1           ;ano je, skok na SepnutSet1
              call Delay100ms           ;ne neni, pocka 100ms
              btfss PORTB,k_set         ;je tlacitko SET stisknuto?
              goto SepnutSet1           ;ano je, skok na SepnutSet1
              call SetNumbers           ;ne uz neni, zavolani nastaveni noveho cisla
              return                    ;navrat z SepnutSet




; Smazani citace a inkrementace adresy ulozeni v EEPROM ----------------------------------

ClearCounter  clrf cnthi                ;smaze cnthi
              clrf cntlo                ;smaze cntlo
              movlw 02h                 ;adresa s pozici (taky adresa) citace v EEPROM
              call ReadEE               ;nacte pozici do W
              movwf temp                ;W do temp
              incf temp,f               ;inkrementuje temp a ulozi do temp
              movlw 7Fh                 ;7F (konec adresniho prostoru EEPROM)
              subwf temp,w              ;odecte temp od 7F
              btfss STATUS,Z            ;je temp=7F?
              goto ClearCounter1        ;ne neni, skok na ClearCounter1
              movlw 4                   ;ano je, adresace je znovu od 4
              movwf temp                ;4 do temp
ClearCounter1 movfw temp                ;temp do W
              bsf STATUS,RP0            ;bank1      
              movwf EEDATA              ;Wcko do EEDATA
              bcf STATUS,RP0            ;bank0      
              movlw 2                   ;adresa do Wcka, tam je adresa s pozici
              bsf STATUS,RP0            ;bank1      
              movwf EEADR               ;Wcko do EEADR
              call ProgEE               ;zapis do EE, do banky 0 prepne ProgEE sam
              call WriteCnt2EE          ;zapis aktualni hodnoty (0) citace do nove adresy
              bcf stav,countdown        ;zruseni odecitani
              call Zobraz               ;zobrazeni nuly
              return                    ;navrat z ClearCounter



; Ulozeni aktualni hodnoty citace do EEPROM ---------------------------------------------

WriteCnt2EE   movlw 2h                  ;adresa s pozici (taky adresa) citace v EEPROM
              call ReadEE               ;nacte pozici do W
              movwf temp                ;W do temp
              movfw cnthi               ;cnthi do W
              bsf STATUS,RP0            ;bank1      
              movwf EEDATA              ;Wcko do EEDATA
              bcf STATUS,RP0            ;bank0      
              movfw temp                ;adresa do Wcka
              bsf STATUS,RP0            ;bank1      
              movwf EEADR               ;Wcko do EEADR
              call ProgEE               ;zapis do EE, do banky 0 prepne ProgEE sam 
              incf temp                 ;inkrementuje pozici o jednicku vys
              movfw cntlo               ;cntlo do W
              bsf STATUS,RP0            ;bank1      
              movwf EEDATA              ;Wcko do EEDATA
              bcf STATUS,RP0            ;bank0      
              movfw temp                ;adresa do Wcka
              bsf STATUS,RP0            ;bank1      
              movwf EEADR               ;Wcko do EEADR
              call ProgEE               ;zapis do EE, do banky 0 prepne ProgEE sam 
              return                    ;navrat z WriteCnt2EE



; Zastaveni stroje a blikajici cislo na displeji -----------------------------------------

MachineStop   movlw 13                  ;napise E
              call Decoder              ;posle do dekoderu
              movwf d100                ;zobrazi v radu 100
              movlw 14                  ;napise n
              call Decoder              ;posle do dekoderu
              movwf d10                 ;zobrazi v radu 10
              movlw 15                  ;napise d
              call Decoder              ;posle do dekoderu
              movwf d1                  ;zobrazi v radu 1
              bsf PORTB,rele            ;sepne rele, stroj off
              call Delay1Sec            ;cas 3 sekundy
              call Delay1Sec
              call Delay1Sec
              bcf PORTB,rele            ;odepne rele, stroj uz musi byt vypnuty
MachineStop1  movlw 12                  ;napise mezeru, tedy v podstate nic
              call Decoder              ;posle do dekoderu    
              movwf d100                ;zobrazi v radu 100
              movlw 12                  ;napise mezeru, tedy v podstate nic
              call Decoder              ;posle do dekoderu
              movwf d10                 ;zobrazi v radu 10
              movlw 12                  ;napise mezeru, tedy v podstate nic
              call Decoder              ;posle do dekoderu
              movwf d1                  ;zobrazi v radu 1
              call Delay100ms           ;pocka 200ms
              call Delay100ms
              btfss PORTB,k_clr         ;je stisknuto tlacitko CLEAR?
              goto MachineStop2         ;ano je, skok na MachineStop2
              movfw stophi              ;kopie stophi
              movwf cnthi               ;do cnthi
              movfw stoplo              ;kopie stoplo
              movwf cntlo               ;do cntlo
              call Zobraz               ;zobrazit obsah cnthi a cntlo
              call Delay100ms           ;pocka 200ms
              call Delay100ms         
              btfsc PORTB,k_clr         ;je stiknuto tlacitko CLEAR?
              goto MachineStop1         ;ne neni, skok na MachineStop1, blikani cisla
MachineStop2  call Delay100ms           ;pocka 100ms
              btfsc PORTB,k_clr         ;je stale stisknuto tlacitko CLEAR?
              goto MachineStop1         ;ne neni, skok na MachineStop1, blikani cisla   
              clrf cnthi                ;ano je, smaze cnthi
              clrf cntlo                ;smaze cntlo
MachineStop3  btfss PORTB,k_clr         ;je stisknuto tlacitko CLEAR?
              goto MachineStop3         ;ano je, skok na MachineStop3
              call Delay100ms           ;pocka 100ms
              btfss PORTB,k_clr         ;je stale stisknuto tlacitlo CLEAR?
              goto MachineStop3         ;ano je, skok na MachineStop3
              call Zobraz               ;ne neni, zobrazeni obsahu chthi, cntlo
              return                    ;navrat z MachineStop




; Natstaveni koncoveho stavu citace ------------------------------------------------------

SetNumbers    bsf stav,cutzeros         ;zakaz orezavani nul a pridavani znamenka
              movfw stophi              ;kopie stophi
              movwf cnthi               ;do cnthi
              movfw stoplo              ;kopie stoplo
              movwf cntlo               ;do cntlo
              call Zobraz               ;defkato jen prevod cnthi a cntlo do D2,D1,D0
              bcf stav,cutzeros         ;povoleni orezavani nul a pridavani znamenka
              ; --- Nastaveni stovek ---
SetN100       movfw D2                  ;D2 do W                   
              call Decoder              ;zavola dekoder, vystup do W
              movwf d100                ;W na displej do radu 100 
              movlw 11                  ;podtrzitko do W          
              call Decoder              ;zavola dekoder, vystup do W
              movwf d10                 ;W na displej do radu 10  
              movwf d1                  ;W na displej do radu 1   
SetN100a      btfss PORTB,k_up          ;je stisknuto tlacitko UP?
              goto SetN100b             ;ano je, skok na goto SetN100b   
              btfss PORTB,k_set         ;je stisknuto tlacitko SET?
              goto SetN100d             ;ano je, skok na goto SetN100d
              goto SetN100a             ;skok na zacatek smycky SetN100a 
SetN100b      call Delay100ms           ;pocka 100ms              
              btfsc PORTB,k_up          ;je stale tlacitko UP stisknute?
              goto SetN100a             ;ne neni, skok na hlavni smycku SetN100a
              incf D2                   ;ano je, inkrementace D2  
              movlw 10                  ;10 do W                  
              subwf D2,W                ;odecte D2 od 10          
              btfsc STATUS,Z            ;je D2=10?                
              clrf D2                   ;ano je, smazat D2        
              movfw D2                  ;D2 do W                  
              call Decoder              ;zavola dekoder, vystup do W
              movwf d100                ;W na displej do radu 100 
SetN100c      btfss PORTB,k_up          ;je stisknuto tlacitko UP?
              goto SetN100c             ;ano, skok na SetN100c    
              call Delay100ms           ;ne neni, skok na Delay100ms 
              btfss PORTB,k_up          ;je stale stisknuto tlacitko UP?
              goto SetN100c             ;ano je, skok na SetN100c 
              goto SetN100a             ;skok na hlavni smycku SetN100a
SetN100d      call Delay100ms           ;pocka 100ms              
              btfsc PORTB,k_set         ;je stale stisknuto tlacitko SET?
              goto SetN100a             ;ne neni, skok na hlavni smycku SetN100a
SetN100e      btfss PORTB,k_set         ;ano je, je stisknuto tlacitko SET?
              goto SetN100e             ;ano je, skok na SetN100e 
              call Delay100ms           ;ne neni, pocka 100ms     
              btfss PORTB,k_set         ;je stale stisknuto tlacitko SET? 
              goto SetN100e             ;ano je, skok na SetN100e                    
              ; --- Nastaveni desitek ---
SetN10        movfw D1                  ;D1 do W                                 
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d10                 ;W na displej do radu 10                
              movlw 11                  ;podtrzitko do W                         
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d100                ;W na displej do radu 100                 
              movwf d1                  ;W na displej do radu 1                  
SetN10a       btfss PORTB,k_up          ;je stisknuto tlacitko UP?               
              goto SetN10b              ;ano je, skok na goto SetN10b           
              btfss PORTB,k_set         ;je stisknuto tlacitko SET?              
              goto SetN10d              ;ano je, skok na goto SetN10d           
              goto SetN10a              ;skok na zacatek smycky SetN10a         
SetN10b       call Delay100ms           ;pocka 100ms                             
              btfsc PORTB,k_up          ;je stale tlacitko UP stisknute?         
              goto SetN10a              ;ne neni, skok na hlavni smycku SetN10a 
              incf D1                   ;ano je, inkrementace D2                 
              movlw 10                  ;10 do W                                 
              subwf D1,W                ;odecte D1 od 10                         
              btfsc STATUS,Z            ;je D1=10?                               
              clrf D1                   ;ano je, smazat D2                       
              movfw D1                  ;D1 do W                                 
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d10                 ;W na displej do radu 10                
SetN10c       btfss PORTB,k_up          ;je stisknuto tlacitko UP?               
              goto SetN10c              ;ano, skok na SetN10c                   
              call Delay100ms           ;ne neni, skok na Delay100ms             
              btfss PORTB,k_up          ;je stale stisknuto tlacitko UP?         
              goto SetN10c              ;ano je, skok na SetN10c                
              goto SetN10a              ;skok na hlavni smycku SetN10a          
SetN10d       call Delay100ms           ;pocka 100ms                             
              btfsc PORTB,k_set         ;je stale stisknuto tlacitko SET?        
              goto SetN10a              ;ne neni, skok na hlavni smycku SetN10a 
SetN10e       btfss PORTB,k_set         ;ano je, je stisknuto tlacitko SET?      
              goto SetN10e              ;ano je, skok na SetN10e                
              call Delay100ms           ;ne neni, pocka 100ms                    
              btfss PORTB,k_set         ;je stale stisknuto tlacitko SET?        
              goto SetN10e              ;ano je, skok na SetN10e                
              ; --- Nastaveni jednotek ---
SetN1         movfw D0                  ;D0 do W                                 
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d1                  ;W na displej do radu 1                
              movlw 11                  ;podtrzitko do W                         
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d100                ;W na displej do radu 100                 
              movwf d10                 ;W na displej do radu 10                  
SetN1a        btfss PORTB,k_up          ;je stisknuto tlacitko UP?               
              goto SetN1b               ;ano je, skok na goto SetN1b           
              btfss PORTB,k_set         ;je stisknuto tlacitko SET?              
              goto SetN1d               ;ano je, skok na goto SetN1d           
              goto SetN1a               ;skok na zacatek smycky SetN1a         
SetN1b        call Delay100ms           ;pocka 100ms                             
              btfsc PORTB,k_up          ;je stale tlacitko UP stisknute?         
              goto SetN1a               ;ne neni, skok na hlavni smycku SetN1a 
              incf D0                   ;ano je, inkrementace D0                 
              movlw 10                  ;10 do W                                 
              subwf D0,W                ;odecte D0 od 10                         
              btfsc STATUS,Z            ;je D0=10?                               
              clrf D0                   ;ano je, smazat D0                       
              movfw D0                  ;D0 do W                                 
              call Decoder              ;zavola dekoder, vystup do W             
              movwf d1                  ;W na displej do radu 1                
SetN1c        btfss PORTB,k_up          ;je stisknuto tlacitko UP?               
              goto SetN1c               ;ano, skok na SetN1c                   
              call Delay100ms           ;ne neni, skok na Delay100ms             
              btfss PORTB,k_up          ;je stale stisknuto tlacitko UP?         
              goto SetN1c               ;ano je, skok na SetN1c                
              goto SetN1a               ;skok na hlavni smycku SetN1a          
SetN1d        call Delay100ms           ;pocka 100ms                             
              btfsc PORTB,k_set         ;je stale stisknuto tlacitko SET?        
              goto SetN1a               ;ne neni, skok na hlavni smycku SetN100a 
SetN1e        btfss PORTB,k_set         ;ano je, je stisknuto tlacitko SET?      
              goto SetN1e               ;ano je, skok na SetN1e                
              call Delay100ms           ;ne neni, pocka 100ms                    
              btfss PORTB,k_set         ;je stale stisknuto tlacitko SET?        
              goto SetN1e               ;ano je, skok na SetN1e                
              ; --- Kontrola jestli neni zadane 000 ---
              movlw 0                   ;0 do W
              subwf D2,W                ;odecte D2 od 0
              btfss STATUS,Z            ;je D2=0?
              goto SetOut               ;ne neni, skok na SetOut 
              movlw 0                   ;0 do W                 
              subwf D1,W                ;odecte D1 od 0         
              btfss STATUS,Z            ;je D1=0?               
              goto SetOut               ;ne neni, skok na SetOut
              movlw 0                   ;0 do W                 
              subwf D0,W                ;odecte D0 od 0         
              btfss STATUS,Z            ;je D0=0?               
              goto SetOut               ;ne neni, skok na SetOut
              goto SetN100              ;vsechno je nula, zadavame znova vsechno
              ; --- Prevod z BCD na bin ---
SetOut        clrf tenk                 ;smaze desetitisice
              clrf D3                   ;smaze tisice
              movf D1, w                ;desitky do W
              addwf D3, w               ;secte tisice s W
              movwf lo                  ;lo = W
              addwf lo, f               ;W = lo + W
              rlf tenk, w               ;rotace desetiticicu do W
              movwf hi                  ;hi = W
              swapf D2, w               ;prehod pulbyty stovek do W
              addwf D2, w               ;W = stovky + W
              addwf lo, f               ;lo = W + lo
              movf D3, w                ;tisice do W
              addwf hi, f               ;hi = W + hi
              rlf lo, f                 ;rotace lo do lo
              rlf hi, f                 ;rotace hi do hi
              swapf D2, w               ;prehod pulbyty stovek do W
              addwf D1, w               ;W = W + desitky
              addwf lo, f               ;lo = lo + W
              swapf tenk, w             ;prehod pulbyty desetitisicu do W       
              skpnc                     ;skip if non carry, preskoci kdyz nebyl prenos C
              iorlw 0x01                ;exclusice OR Wcka konstantou 1 do W
              addwf hi, f               ;hi = W + hi
              swapf D3, w               ;prehod pulbyty tisicu do W    
              subwf lo, f               ;lo = W + lo
              skpc                      ;skip if carry, preskoci kdyz byl prenos C
              decf hi, f                ;hi = hi - 1
              clrc                      ;clear carry, smaze priznak prenosu C
              rlf lo, f                 ;rotace lo do lo
              rlf hi, f                 ;rotace hi do hi
              swapf tenk, w             ;prehod pulbyty desetitisicu do W   
              addwf D0, w               ;W = W + jednotky
              addwf lo, f               ;lo = W + lo
              movf tenk, w              ;tisice do W (movf ovlivnuje carry, resp. STATUS)
              skpnc                     ;skip if non carry, preskoci kdyz nebyl prenos C
              decf tenk, w              ;W = desetitisice - 1
              subwf hi, f               ;hi = W + hi
              ; --- Ulozeni do EEPROM vysledku z hi,lo ---
              movfw hi                  ;vysledek z hi do W
              bsf STATUS,RP0            ;bank1      
              movwf EEDATA              ;Wcko do EEDATA
              bcf STATUS,RP0            ;bank0      
              movlw 0h                  ;adresa do Wcka
              bsf STATUS,RP0            ;bank1      
              movwf EEADR               ;Wcko do EEADR
              call ProgEE               ;zapis do EE, do banky 0 prepne ProgEE sam 
              movfw lo                  ;vysledek z lo do W
              bsf STATUS,RP0            ;bank1      
              movwf EEDATA              ;Wcko do EEDATA
              bcf STATUS,RP0            ;bank0      
              movlw 1h                  ;adresa do Wcka
              bsf STATUS,RP0            ;bank1      
              movwf EEADR               ;Wcko do EEADR
              call ProgEE               ;zapis do EE, do banky 0 prepne ProgEE sam 
              ; --- Aktualizace stophi a stoplo ---
              movfw hi                  ;kopie vysledku hi
              movwf stophi              ;do stophi
              movfw lo                  ;kopie vysledku lo
              movwf stoplo              ;do stoplo
              call ClearCounter         ;smaze citac, ulozi do EEPROM apod..
              return                    ;navrat ze SetNumbers






; Prevod bin na BCD a pripadny rozdil ----------------------------------------------------

Zobraz              ; --- Ulozeni cnt do pomocnych promennych ---
              movfw cnthi                     ;kopie cnthi
              movwf NUMHI               ;do pracovni NUMHI
              movfw cntlo               ;kopie cntlo
              movwf NUMLO               ;do pracovni NUMLO
              btfss stav,countdown        ;ma se zobrazovat odecet?
                    ; --- Provedeni rozdilu, pokud ma byt proveden ---
              goto Zobraz2              ;nema, skok na zobrazeni
              movfw stophi              ;kopie stophi
              movwf CIS1_HI             ;do CIS1_HI
              movfw stoplo              ;kopie stoplo
              movwf CIS1_LO             ;do CIS1_LO
              movfw cnthi               ;kopie cnthi
              movwf CIS2_HI             ;do CIS2_HI
              movfw cntlo               ;kopie cntlo
              movwf CIS2_LO             ;do CIS2_LO
              movf CIS2_LO, W           ;CISLO_2 do W
              subwf CIS1_LO, W          ;W = CISLO1_LO - CISLO2_LO
              movwf NUMLO               ;NUMLO = W
              btfss STATUS, C           ;doslo k preteceni, resp. nastaveni C?
              decf CIS1_HI              ;ne, odecti jednicku od CIS1_HI
              movf CIS2_HI, W           ;ano+ne, W=CIS2_HI 
              subwf CIS1_HI, W          ;W = CISLO1_HI - CISLO2_HI
              movwf NUMHI               ;NUMHI = W
                    ; --- Prevod bin na BCD (D3,D2,D1,D0) ---
Zobraz2       movf NUMHI,w              ;w=NUMHI
              iorlw 0xF0                ;w=H2-16
              movwf D1                  ;D1=H2-16
              addwf D1,f                ;D1=H2*2-32
              addwf D1,f                ;D1=H2*3-48
              movwf D2                  ;D2=H2-16
              addlw -D'5'               ;w=H2-21
              addwf D2,f                ;D2=H2*2-37 Done!
              addlw D'41'               ;w=H2+20
              movwf D0                  ;D0=H2+20
              swapf NUMLO,w             ;prehod horni a spodni polovinu NUMLO a do W
              iorlw 0xF0                ;w=H1-16
              addwf D1,f                ;D1=H2*3+H1-64
              addwf D0,f                ;D0=H2+H1+4, C=1
              rlf D0,f                  ;D0=(H2+H1)*2+9, C=0
              comf D0,f                 ;D0=-(H2+H1)*2-10
              rlf D0,f                  ;D0=-(H2+H1)*4-20
              movf NUMLO,w              ;NUMLO do W
              andlw 0x0F                ;w=H0
              addwf D0,f                ;D0=H0-(H2+H1)*4-20 Done!
              rlf D1,f                  ;C=0, D1=H2*6+H1*2-128 Done! 
              movlw D'5'                ;dej 5
              movwf D3                  ;do D3
              movlw D'10'               ;do W nastav 10  
Zobrazmod0    addwf D0,f                ;D(X)=D(X)mod10
              decf D1,f                 ;D(X+1)=D(X+1)+D(X)div10
              skpc                      ;skip if carry, preusoinstrukce MPASM  
              goto Zobrazmod0           ;skok na Zobrazmod0
Zobrazmod1    addwf D1,f                ;soucet D1 s W, vysledek do D1
              decf D2,f                 ;odecte 1 od D2, vysledek do D2
              skpc                      ;skip if carry, preusoinstrukce MPASM  
              goto Zobrazmod1           ;skok na Zobrazmod1
Zobrazmod2    addwf D2,f                ;soucet D2 s W, vysledek do D2
              decf D3,f                 ;odecte 1 od D3, vysledek do D3
              skpc                      ;skip if carry, preusoinstrukce MPASM  
              goto Zobrazmod2           ;skok na Zobrazmod2
                    ; --- Uprava vysledneho zobrazeni vysledku z D2,D1,D0 ---
              btfsc stav,cutzeros       ;ma dojit k orezani nul a pridani znamenka? 
              goto ZobrazP1             ;nema, skok na prime zobrazeni vseho      
                    ; --- Orezani nul ---
              movlw 0                   ;0 do W
              subwf D2,W                ;odecet D2 od 0 a vysledek do W
              btfss STATUS,Z            ;je v D2 nula?
              goto ZobrazP3             ;neni, neni co rezat, skok na znamenko..
              movlw 12                  ;ano, pridani 12(mezera dle dekoderu) 
              movwf D2                  ;do D2 - na displeji v radu 100 nebude nic
              movlw 0                   ;0 do W
              subwf D1,W                ;odecet D1 od 0 a vysledek do W
              btfss STATUS,Z            ;je v D1 nula?
              goto ZobrazP3             ;neni, neni co rezat, skok na znamenko..
              movlw 12                  ;ano, pridani 12(mezera dle dekoderu) 
              movwf D1                  ;do D1 - na displeji v radu 10 nebude nic
                    ; --- Pridani znamenka, ma-li se pridat a vleze-li se ---
ZobrazP3      btfss stav,countdown      ;ma se zobrazovat odecet?
              goto ZobrazP1             ;nema, skok na zobrazeni toho co v D2,1,0 je
              movlw 12                  ;ano ma, 12 do W 
              subwf D1,W                ;odecet D1 od 12, vysledek do W
              btfss STATUS,Z            ;je v D1 hodnota 12?
              goto ZobrazP2             ;ne neni, skok na Zobraz P2
              movlw 10                  ;ano je, pridani 10(pomlcka dle dekoderu) 
              movwf D1                  ;do D1 - na displeji v radu 10 bude pomlcka
              goto ZobrazP1             ;jestlize je v radu 10 -, v radu 100 uz nebude
ZobrazP2      movlw 12                  ;ano ma, 12 do W 
              subwf D2,W                ;odecet D2 od 12, vysledek do W
              btfss STATUS,Z            ;je v D2 hodnota 12?
              goto ZobrazP1             ;ne neni, skok na Zobraz P2
              movlw 10                  ;ano je, pridani 10(pomlcka dle dekoderu) 
              movwf D2                  ;do D2 - na displeji v radu 100 bude pomlcka
                    ; --- Prevod BCD na sedmisegmentovku ---
ZobrazP1      movfw D2                  ;D2 do W  
              call Decoder              ;zavolani dekoderu, vystup bude ve W
              movwf d100                ;W do radu 100 na displeji
              movfw D1                  ;D1 do W  
              call Decoder              ;zavolani dekoderu, vystup bude ve W
              movwf d10                 ;W do radu 10 na displeji
              movfw D0                  ;D0 do W  
              call Decoder              ;zavolani dekoderu, vystup bude ve W
              movwf d1                  ;W do radu 1 na displeji
              return                    ;navrat z podprogramu Zobraz
            
                  
 

; Kod pro praci s EEPROM -----------------------------------------------------------------

      ; Cteni z EEPROM. Vstupem je adresa ve W a vystupem jsou data ve W
ReadEE        bsf STATUS,RP0             ;bank 1
              movwf EEADR                ;W do EEADR
              bsf EECON1,RD              ;zahajeni cteni
ReadEE1       btfsc EECON1,RD            ;je RD bit log.1?
              goto ReadEE1               ;neni
              movfw EEDATA               ;EEDATA do W
              bcf STATUS,RP0             ;bank 0
              return                     ;navrat

      ; Zapis do EEPROM. Vstupem je nastavena bank1 a naplnene EEDATA a EEADR
ProgEE        bsf EECON1,WREN            ;povoleni zapisu
              bcf INTCON,GIE             ;zakaze vsechna preruseni
              movlw 55h                  ;povinna sekvence
              movwf EECON2
              movlw 0AAh
              movwf EECON2
              bsf EECON1,WR              ;zapis do EE
              bsf INTCON,GIE             ;povoli vsechna preruseni
ProgEE1       btfsc EECON1,WR            ;je WR bit log.1?
              goto ProgEE1               ;neni
              bcf EECON1,WREN            ;zakaz zapisu zapisu
              bcf STATUS,RP0             ;bank 0
              return                     ;navrat




; Kod pro generovani casovych smycek -----------------------------------------------------

Delay1Sec     ;return
              movlw 0FFh                ;255 do W
              movwf count               ;W do count
DelayLoop     call Pause                ;pauza
              call Pause
              call Pause
              call Pause
              call Pause
              decfsz count,f            ;dekrementace count a preskok, pokud count=0
              goto DelayLoop            ;skok na DelayLoop
              return                    ;navrat
             
Delay300ms    ;return
              call Delay100ms           ;pocka 100ms
              call Delay100ms 
              call Delay100ms 
              return                    ;navrat

Delay100ms    ;return
              movlw 25                  ;25 do W
              movwf count               ;W do count
DelayL100     call Pause                ;pauza
              call Pause
              call Pause
              call Pause
              call Pause
              decfsz count,f            ;dekrementace count a preskok, pokud count=0
              goto DelayL100            ;skok na DelayL100
              return                    ;navrat

Pause         ;return
              movlw 0FFh                ;255 do W
              movwf pau                 ;W do pau
PauseLoop     clrwdt                    ;smaze watchdog
              decfsz pau,f              ;dekrementace pau a preskok, pokud pau=0
              goto PauseLoop            ;skok na PauseLoop
              return                    ;navrat




 end
